package constants

import "fmt"

const WORKER_VERSION = "v1"

const WORKER_VERSION_HEADER = "morffix-version"
const SHARED_SECRET_HEADER = "morffix-secret"
const NAME_HEADER = "morffix-name"
const UUID_HEADER = "morffix-uuid"
const HASH_HEADER = "morffix-hash"

const INDEX_TEMPLATE_NAME = "index.tmpl"
const LIBRARIES_TEMPLATE_NAME = "libraries.tmpl"
const LIBRARY_TEMPLATE_NAME = "library.tmpl"
const MESSAGE_TEMPLATE_NAME = "message.tmpl"
const TASKS_TEMPLATE_NAME = "tasks.tmpl"
const TASK_TEMPLATE_NAME = "task.tmpl"
const FFMPEG_COMMANDS_TEMPLATE_NAME = "ffmpeg_commands.tmpl"
const STATS_TEMPLATE_NAME = "stats.tmpl"

const FORM_FILE_KEY = "file"

const SORT_ORDER_BY_PARAM = "order_by"
const SORT_ORDER_ASC_PARAM = "order_asc"
const QUERY_LIMIT_PARAM = "limit"
const QUERY_PAGE_PARAM = "page"

type TaskType int

const (
	TASK_TYPE_HEALTHCHECK TaskType = iota
	TASK_TYPE_TRANSCODE
)

func (s TaskType) String() string {
	switch s {
	case TASK_TYPE_HEALTHCHECK:
		return "Healthcheck"
	case TASK_TYPE_TRANSCODE:
		return "Transcode"
	default:
		return fmt.Sprintf("%d", int(s))
	}
}

type TaskStatus int

// Non Append Changes Need Worker Version Bump
const (
	TASK_STATUS_UNKNOWN TaskStatus = iota
	TASK_STATUS_FAILED
	TASK_STATUS_SUCCESS
	TASK_STATUS_RUNNING
	TASK_STATUS_QUEUED
	TASK_STATUS_ASSIGNED
	TASK_STATUS_PAUSED
	TASK_STATUS_WAITING
)

func (s TaskStatus) String() string {
	switch s {
	case TASK_STATUS_UNKNOWN:
		return "Unknown"
	case TASK_STATUS_FAILED:
		return "Failed"
	case TASK_STATUS_SUCCESS:
		return "Success"
	case TASK_STATUS_RUNNING:
		return "Running"
	case TASK_STATUS_QUEUED:
		return "Queued"
	case TASK_STATUS_ASSIGNED:
		return "Assigned"
	case TASK_STATUS_PAUSED:
		return "Paused"
	case TASK_STATUS_WAITING:
		return "Waiting"
	default:
		return fmt.Sprintf("%d", int(s))
	}
}

type FileStatus int

const (
	FILE_STATUS_UNKNOWN FileStatus = iota
	FILE_STATUS_MISSING
	FILE_STATUS_EXISTS
	FILE_STATUS_CHANGED
	FILE_STATUS_NEW
)

func (s FileStatus) String() string {
	switch s {
	case FILE_STATUS_UNKNOWN:
		return "Unknown"
	case FILE_STATUS_MISSING:
		return "Missing"
	case FILE_STATUS_EXISTS:
		return "Exists"
	case FILE_STATUS_CHANGED:
		return "Changed"
	case FILE_STATUS_NEW:
		return "New"
	default:
		return fmt.Sprintf("%d", int(s))
	}
}

type FileHealth int

const (
	FILE_HEALTH_UNKNOWN FileHealth = iota
	FILE_HEALTH_DAMAGED
	FILE_HEALTH_HEALTHY
)

func (s FileHealth) String() string {
	switch s {
	case FILE_HEALTH_UNKNOWN:
		return "Unknown"
	case FILE_HEALTH_DAMAGED:
		return "Damaged"
	case FILE_HEALTH_HEALTHY:
		return "Healthy"
	default:
		return fmt.Sprintf("%d", int(s))
	}
}

type FileTranscode int

const (
	FILE_TRANSCODE_UNKNOWN FileTranscode = iota
	FILE_TRANSCODE_NONE
	FILE_TRANSCODE_FAILED
	FILE_TRANSCODE_SUCCESS
)

func (s FileTranscode) String() string {
	switch s {
	case FILE_TRANSCODE_UNKNOWN:
		return "Unknown"
	case FILE_TRANSCODE_NONE:
		return "None"
	case FILE_TRANSCODE_FAILED:
		return "Failed"
	case FILE_TRANSCODE_SUCCESS:
		return "Success"
	default:
		return fmt.Sprintf("%d", int(s))
	}
}
